/*
 *  client.rs
 *
 *  Copyright (C) 2020 Qarva
 *  Sun Dro <skalatozishvili@qarva.com>
 *
 * Qarva aQua Web Multipipe client.
 */

extern crate websocket;
extern crate json;

use crate::qws::packet;
use websocket::WebSocketResult;
use websocket::sync::Client;
use std::net::TcpStream;

pub struct QClient
{
    stream: Client<TcpStream>,
    connects: usize,
    connect: usize,
    content: u32,
    time: String
}

impl QClient {

    pub fn connect(addr: &str) -> QClient {
        QClient {
            stream: websocket::ClientBuilder::new(addr).unwrap()
                .add_protocol("qarva-ws")
                .connect_insecure().unwrap(),
            connects: 1,
            connect: 1,
            content: 0,
            time: "0".to_string()
        }
    }

    pub fn set_connect(&mut self, cnt: usize, count: usize) {
        self.connects = count;
        self.connect = cnt;
    }

    pub fn set_content(&mut self, content: u32, time: &str) {
        self.content = content;
        self.time = time.to_string();
    }

    pub fn send_command(&mut self, cmd: &str) -> WebSocketResult<()> {
        let header: String;
        if cmd.to_string() == "play" {
            header = json::object! {
                "contentId" => self.content,
                "connects" => self.connects,
                "connect" => self.connect,
                "time" => self.time.clone(),
                "cmd" => cmd.to_string()
            }.dump();
        } else {
            header = json::object! {
                "cmd" => cmd.to_string()
            }.dump();
        }

        let message = websocket::Message::text(&header);
        self.log_info(">", header.clone());
        self.stream.send_message(&message)
    }

    pub fn recv_packet(&mut self) -> packet::QPacket {
        let frame = self.stream.recv_dataframe().unwrap();
        let mut packet = packet::QPacket::parse(frame.data);
        self.log_info("<", packet.get_header());
        packet.set_frag_num(self.connect as u16);
        packet
    }

    pub fn log_info(&mut self, cur: &str, header: String) {
        println!("c({})[{}] {} {}", 
            self.connects, 
            self.connect, 
            cur, header);
    }
}
